from __future__ import absolute_import

import json
import os
import yaml
import logging
import requests
from typing import List, Dict

logger = logging.getLogger(__name__)
logging.basicConfig()


# https://stackoverflow.com/questions/6760685/creating-a-singleton-in-python
class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if 'verbose' in kwargs and kwargs['verbose']:
            logger.setLevel(logging.INFO)
        else:
            logger.setLevel(logging.ERROR)

        if cls not in cls._instances:
            logger.info('creating a new aoa client object')
            cls._instances[cls] = super(Singleton, cls).__call__(
                *args,
                **kwargs)
        else:
            logger.info('using an existing aoa client object')
        return cls._instances[cls]


class AoaClient(metaclass=Singleton):
    def __init__(self, configuration=None,
                 verbose=False,
                 **kwargs):

        self.project_id = None
        self.auth = None, None
        self.aoa_url = None

        self.__parse_aoa_config(**kwargs)

    def __parse_yaml(self, yaml_path: str):
        with open(yaml_path, "r") as handle:
            conf = yaml.safe_load(handle)
        self.__parse_kwargs(**conf)

    def __parse_kwargs(self, **kwargs):
        self.project_id = kwargs["project_id"]
        self.auth = kwargs["auth_user"], kwargs["auth_pass"]
        self.aoa_url = kwargs["aoa_url"]
        if "verify_connection" in kwargs:
            self.verify_aoa_connection = kwargs["verify_connection"]

    def __parse_env_variables(self):
        self.project_id = os.environ["AOA_PROJECT_ID"]
        self.auth = os.environ["AOA_API_AUTH_USER"], os.environ["AOA_API_AUTH_PASS"]
        self.aoa_url = os.environ["AOA_URL"]

    def set_project_id(self, project_id: str):
        """
        set project id

        Parameters:
           project_id (str): project id(uuid)
        """
        self.project_id = project_id

    def get_current_project(self):
        """
        get project id

        Return:
           project_id (str): project id(uuid)
        """
        return self.project_id

    def __parse_aoa_config(self, **kwargs):
        if "conf_file" in kwargs:
            self.__parse_yaml(kwargs['conf_file'])
        elif "parse_env_variables" in kwargs and kwargs["parse_env_variables"]:
            self.__parse_env_variables()
        else:
            self.__parse_kwargs(**kwargs)

    def select_header_accept(self, accepts: List[str]):
        """
        converts list of header into a string

        Return:
            (str): request header
        """
        if not accepts:
            return
        accepts = [x.lower() for x in accepts]
        if 'application/json' in accepts:
            return 'application/json'
        else:
            return ', '.join(accepts)

    def get_request(self, path, header_params: Dict[str, str], query_params: Dict[str, str]):
        """
        wrapper for get request

        Parameters:
           path (str): url
           header_params (dict): header parameters
           query_params (dict): query parameters

        Returns:
            str or dict
        """
        resp = requests.get(
            url=self.aoa_url + path,
            auth=self.auth,
            headers=header_params,
            params=query_params
        )
        try:
            return resp.json()
        except ValueError:
            return resp.text

    def post_request(self, path, header_params: Dict[str, str], query_params: Dict[str, str], body: Dict[str, str]):
        """
        wrapper for post request

        Parameters:
           path (str): url
           header_params (dict): header parameters
           query_params (dict): query parameters
           body (dict): request body

        Returns:
            str or dict
        """
        resp = requests.post(
            url=self.aoa_url + path,
            auth=self.auth,
            headers=header_params,
            params=query_params,
            data=json.dumps(body)
        )
        try:
            return resp.json()
        except ValueError:
            return resp.text

    def put_request(self, path, header_params: Dict[str, str], query_params: Dict[str, str], body: Dict[str, str]):
        """
        wrapper for put request

        Parameters:
           path (str): url
           header_params (dict): header parameters
           query_params (dict): query parameters
           body (dict): request body

        Returns:
            str or dict
        """
        resp = requests.put(
            url=self.aoa_url + path,
            auth=self.auth,
            headers=header_params,
            params=query_params,
            data=json.dumps(body)
        )
        try:
            return resp.json()
        except ValueError:
            return resp.text
