import json
import uuid
import collections
import os
import shutil
from pathlib import Path
import logging

logging.root.setLevel(logging.DEBUG)


class ModelUtility(object):

    def __init__(self, model_catalog_path: str, model_template_path: str = None):
        self.dir_path, _ = os.path.split(__file__)
        self.template_catalog = os.path.join(self.dir_path, 'model_templates/')
        self.model = collections.OrderedDict()
        self.model_catalog_path = os.path.join(model_catalog_path, '')
        self.model_template = "empty"
        self.model_template_path = os.path.join(model_template_path, '') if model_template_path is not None else None
        self.catalog, self.custom_catalog = self.get_all_template_catalog(model_template_path)

    def add_model(self, model_name: str, model_desc: str, lang: str,
                  template: str, base_path: str = None, model_template_path: str = None):

        base_path = self.model_catalog_path if base_path is None else os.path.join(base_path, '')
        model_path = os.path.join(base_path + 'model_definitions/', '')
        model_template_path = self.model_template_path if model_template_path is None \
            else os.path.join(model_template_path, '')

        if not os.path.isdir(model_path):
            raise ValueError("model directory {0} does not exist".format(model_path))

        self.model["id"] = str(uuid.uuid4())
        self.model["name"] = model_name
        self.model["description"] = model_desc
        self.model["language"] = lang

        self.model_template = template

        self.__add_framework_specific_attributes()
        self.__create_model_structure(model_path, model_template_path)

    @staticmethod
    def init_model_directory(path: str = None):
        logging.info('creating model directory')
        if path is None:
            path = os.path.join(os.path.abspath(os.getcwd()), '')

        logging.info('creating model definitions')

        src = os.path.join(os.path.split(__file__)[0], '') + 'metadata_files'
        src_files = os.listdir(src)
        for file_name in src_files:
            full_file_name = os.path.join(src, file_name)
            if os.path.isfile(full_file_name) and not os.path.exists(os.path.join(path, file_name)):
                shutil.copy(full_file_name, path)

        Path(path + "model_definitions/").mkdir(parents=True, exist_ok=True)
        if not os.path.isdir(path + "model_templates/"):
            logging.info('creating model templates')
            shutil.copytree(os.path.join(os.path.split(__file__)[0], '') + 'model_templates',
                            path + 'model_templates')
        else:
            logging.info('model templates already exists...skipping')

    def __add_framework_specific_attributes(self):
        if self.model_template == "pyspark":
            self.model["automation"] = {
                "trainingEngine": "pyspark"
            }

    def __create_model_structure(self, model_path, model_template_path: str = None):
        model_dir = model_path + self.model["id"]
        if model_template_path is None:
            template_dir = self.template_catalog + self.model["language"] + "/" + self.model_template
        else:
            template_dir = model_template_path + self.model["language"] + "/" + self.model_template

        shutil.copytree(template_dir, model_dir)

        with open(model_dir + "/model.json", 'w') as f:
            json.dump(self.model, f, indent=4)

    def get_all_template_catalog(self, template_path: str = None):
        cat, custom_cat = {}, {}

        if template_path is None:
            pass
        else:
            custom_cat = self.__get_template_catalog(template_path)

        cat = self.__get_template_catalog(self.template_catalog)

        return cat, custom_cat

    def __get_template_catalog(self, model_template_path):
        catalog = {}

        sub_folders = [f for f in os.listdir(model_template_path)
                       if os.path.isdir(os.path.join(model_template_path, f))
                       & (f[0] != '.') & (f.lower() != 'readme.md')
                       ]

        for language in sub_folders:
            language_dir = os.path.join(model_template_path, language)
            catalog[language] = []
            for template_type in os.listdir(language_dir):
                if template_type[0] != '.' and template_type.lower() != 'readme.md':
                    catalog[language].append(template_type)
        return catalog







